/*

    this source file is a part of 
    lp_parse - a preprocessor for MILP models
    Copyright (C) 1996  Ari Jolma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    for more information please contact 
    Ari Jolma
    Helsinki University of Technology
    Laboratory of Water Resources Engineering
    Building W
    02150 Espoo, Finland
    E-mail: Ari.Jolma@hut.fi

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "util.h"
#include "lp_index.h"
#include "hash.h"

void AddINDEX(INDEX **root, INDEX *I) {
  I->next = *root;
  *root = I;
}

void RemoveFromINDEXList(INDEX **root, int n) {
  INDEX *tmp;
  int i = 0;
  while (root && (i < n)) {
    tmp = *root;
    *root = (*root)->next;

#ifdef DEBUG
    /*printf("removing from index list %i %s\n", n, tmp->name);*/
#endif

    free(tmp);
    i++;
  }
}

INDEX *NewINDEX(void) {
  INDEX *i = XMS(INDEX);
  if (!i) return NULL;
  i->name = NULL;
  i->has_value = FALSE;
  i->value = 0;
  i->has_bounds = FALSE;
  i->min = 0;
  i->max = 0;
  i->smin = NULL;
  i->smax = NULL;
  i->next = NULL;
  return i;
}

void DeleteINDEXList(INDEX **root) {
  INDEX *tmp;
  while (*root) {
    tmp = *root;
    *root = (*root)->next;
    free(tmp);
  }
}

INDEX *SearchINDEX(INDEX *root, char *name) {
  while (root) {
    if (!strcmp(root->name, name)) return root;
    root = root->next;
  }
  return NULL;
}

int INDEXListToMin(INDEX *root, int n) {
  int i = 0;
  while (root && (i<n)) {
    if (root->min > root->max) return FALSE;
    root->has_value = TRUE;
    root->value = root->min;
    root = root->next;
    i++;
  }
  return TRUE;
}

int LoopINDEXList(INDEX *root, int n) {
  int i = 0;
  while (root && (i<n)) {
    root->value++;
    if (root->value > root->max) {
      root->value = root->min;
      root = root->next;
      i++;
    } else return TRUE;
  }
  return FALSE;
}

void PrintIndexes(INDEX *root) {
  while (root) {
    printf("%s has value: %i %i has bounds: %i %i %i\n", 
	   root->name, root->has_value, root->value,
	   root->has_bounds, root->min, root->max);
    root = root->next;
  }
}

/*---------------------------------------------------------------------------*/

CONSTANT *NewCONSTANT() {
  CONSTANT *c = XMS(CONSTANT);
  if (!c) return NULL;
  c->name = NULL;
  c->has_value = FALSE;
  c->value = 0;
  c->next = NULL;
  return c;
}

void AddCONSTANT(CONSTANT **root, CONSTANT *C) {
  CONSTANT **tmp;
  if ((*root) == NULL)
    (*root) = C; 
  else {
    tmp = root;
    while ((*tmp)->next != NULL)
      tmp = (CONSTANT **)(&((*tmp)->next));
    (*tmp)->next = C;
  }
}

#define HASH_TABLE_SIZE 256
int use_hash = FALSE;

CONSTANT *SearchCONSTANT(CONSTANT *root, char *name) {
  if (!use_hash) {
    while (root) {
      if (!strcmp(root->name, name)) return root;
      root = root->next;
    }
    return NULL;
  } else {

#ifdef DEBUG
    /*printf("sc %s\n", name);*/
#endif

    return (CONSTANT *)
      search_hash_entry(name,
			(HASH_ENTRY **)root, 
			ATJHash(name) % HASH_TABLE_SIZE);

  }
}

void HashCONSTANT_list(CONSTANT **root) {

  CONSTANT *c = *root, *d;

  (*root) = xcalloc(HASH_TABLE_SIZE, sizeof(HASH_ENTRY *));
  
  while (c) {
    insert_hash_entry((HASH_ENTRY *)(c),
		      (HASH_ENTRY **)(*root), 
		      ATJHash(c->name) % HASH_TABLE_SIZE);
    d = c;
    c = c->next;
    d->next = NULL;
  }
  use_hash = TRUE;

}

void print_hash(CONSTANT *root) {
  HASH_ENTRY **hash_table;
  HASH_ENTRY *h;
  int i,j;
  hash_table = (HASH_ENTRY **)(root);
  for (i=0;i<HASH_TABLE_SIZE;i++) {
    j=0;
    h = hash_table[i];
    while (h) {
      j++;
      h = h->next;
    }
    printf("%i %i\n",i,j);
  }
}

void PrintConstant(CONSTANT *root) {
  while (root) {
    printf("%s %i %f\n", root->name, root->has_value, root->value);
    root = root->next;
  }
}
